<?php
/*
Plugin Name: Blip Widget
Plugin URI: http://blog.greenek.com/
Description: Widget wyswietlajacy ostatnie wpisy uzytkownika z serwisu Blip.pl. Swietnie nadaje sie jako zamiennik minibloga.
Version: 0.2
Author: Greenek
Author URI: http://blog.greenek.com/
*/

function get_recent_blips($options = array()) {
    $username = (isset($options['username'])) ? $options['username'] : 'blipinfo';
    $limit = (isset($options['limit'])) ? $options['limit'] : 5;
    $cache_lifetime = (isset($options['cache_lifetime'])) ? $options['cache_lifetime'] : 0;
    $html = (isset($options['type']) AND $options['type'] == 'html') ? TRUE : FALSE;

    $blips = '';

    if ($cache_lifetime > 0)
    {
        $last_cache = get_option('blip_widget_last_cache');
        $next_cache = $last_cache + ($cache_lifetime * 60);

        $blips = get_option('blip_widget_cache');
    }

    if ($blips == '' OR ($cache_lifetime > 0 AND date('U') > $next_cache))
    {
        $url = 'http://'.$username.'.blip.pl/feed';

        $feed = '';
        $handle = @fopen($url, "r");
        if ($handle)
        {
            while( ! feof($handle))
            {
                $feed .= fgets($handle, 4096);
            }
            fclose($handle);
        }

        if (empty($feed))
            return false;

        $xml = simplexml_load_string($feed);

        if ($xml === false)
        {
            echo '<p>Wygląda na to, że Blip.pl jest w tym momencie przeciążony.</p>';
            return false;
        }

        $i = 1;

        foreach ($xml->entry as $entry)
        {
            if ($html)
            {
                $patterns = array(
                    '/<a href="([[:alnum:]:@#%\&_=?\/\.-]+)" title="Statusy oznaczone tagiem: ([[:alnum:]]+)">/',
                    '/<div class="photo"> <a href="([[:alnum:]:@#%\&_=?\/\.-]+)"><img src="([[:alnum:]:@#%\&_=?\/\.-]+)" alt="default picture" \/><\/a><\/div>/',
                    '/<div class="video youtube">(.+)youtube.com\/v\//s',
                    '/" type="application\/x-shockwave-flash"(.+)<\/div>/s'
                );

                $replacements = array(
                    '<a href="\\1" title="Statusy oznaczone tagiem: \\2" class="blip-tag">',
                    ' <a href="\\1" class="blip-photo">[Foto]</a>',
                    ' <a href="http://www.youtube.com/watch?v=',
                    '" class="blip-video">[YouTube]</a>'
                );

                $content = preg_replace($patterns, $replacements, $entry->content);
            }
            else
            {
                $content = $entry->title;
            }

            $blips .= '<li><em class="blip-date"><a href="'.$entry->link['href'].'">'.date('H:i, d.m', strtotime($entry->updated.' + 1 hour')).'</a></em> <span class="blip-entry">'.$content.'</span></li>';

            if (++$i > $limit)
                break;
        }

        if ($cache_lifetime > 0)
        {
            update_option('blip_widget_last_cache', date('U'));
            update_option('blip_widget_cache', $blips);
        }
    }

    echo $blips;
}

class Blip_Widget extends WP_Widget {

    function Blip_Widget()
    {
        $widget_ops = array('classname' => 'widget_blip', 'description' => 'Ostatnie statusy z serwisu blip.pl');
        $control_ops = array();

        parent::WP_Widget(false, $name = 'Blip.pl', $widget_ops);
    }

    function widget($args, $instance)
    {
        extract($args);

        $title = esc_attr($instance['title']);

        $options = array(
            'username' => esc_attr($instance['username']),
            'limit' => esc_attr($instance['limit']),
            'cache_lifetime' => esc_attr($instance['cache_lifetime']),
            'type' => esc_attr($instance['type'])
        );

        echo $before_widget;
        if ( !empty($title) ) { echo $before_title . $title . $after_title; }
            echo '<ul id="blip-widget">';
            get_recent_blips($options);
            echo '</ul>';
        echo $after_widget;
    }

    function update($new_instance, $old_instance)
    {
        update_option('blip_widget_cache', '');
        return $new_instance;
    }

    function form($instance)
    {
        $instance = wp_parse_args( (array) $instance, array ( 'title' => 'Blip.pl', 'username' => '', 'limit' => '5', 'cache_lifetime' => '0', 'type' => 'html' ) );
        $title = esc_attr($instance['title']);
        $username = esc_attr($instance['username']);
        $limit = esc_attr($instance['limit']);
        $cache_lifetime = esc_attr($instance['cache_lifetime']);
        $type = esc_attr($instance['type']);
        ?>
            <p><label for="<?php echo $this->get_field_id('title') ?>"><?php _e('Title:') ?> <input class="widefat" id="<?php echo $this->get_field_id('title') ?>" name="<?php echo $this->get_field_name('title') ?>" type="text" value="<?php echo $title ?>" /></label></p>

            <p><label for="<?php echo $this->get_field_id('username') ?>">Nazwa użytkownika: <input class="widefat" id="<?php echo $this->get_field_id('username') ?>" name="<?php echo $this->get_field_name('username') ?>" type="text" value="<?php echo $username ?>" /></label></p>

            <p><label for="<?php echo $this->get_field_id('limit') ?>">Limit wpisów: <select class="widefat" id="<?php echo $this->get_field_id('limit') ?>" name="<?php echo $this->get_field_name('limit') ?>" >
            <?php for($i = 1; $i <= 20; $i++): ?>
                <?php $selected = ($i == $limit) ? ' selected="selected"' : '' ?>
                <option value="<?php echo $i ?>"<?php echo $selected ?>><?php echo $i ?></option>
            <?php endfor ?>
            </select></p>

            <p><label for="<?php echo $this->get_field_id('cache_lifetime') ?>">Długość przechowywania cachu w pamięci (w min.). 0 - brak cachu: <input class="widefat" id="<?php echo $this->get_field_id('cache_lifetime') ?>" name="<?php echo $this->get_field_name('cache_lifetime') ?>" type="text" value="<?php echo $cache_lifetime ?>" /></label></p>

            <p><label for="<?php echo $this->get_field_id('type') ?>">Wyświetlaj jako: <select class="widefat" id="<?php echo $this->get_field_id('type') ?>" name="<?php echo $this->get_field_name('type') ?>" >
                <option value="html" <?php if ($type == 'html') echo 'selected="selected"' ?>>HTML</option>
                <option value="plain" <?php if ($type == 'plain') echo 'selected="selected"' ?>>Tekst</option>
            </select></p>
        <?php
    }

}

add_action('widgets_init', create_function('', 'return register_widget("Blip_Widget");'));

?>
